"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Colors = void 0;
var _scheme = _interopRequireDefault(require("./scheme.js"));
var _colorsPalette = require("./colorsPalette.js");
var _designTokens = require("./designTokens.js");
var _designTokensDM = require("./designTokensDM.js");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
class Colors {
  constructor() {
    const colors = Object.assign(_colorsPalette.colorsPalette);
    Object.assign(this, colors);
    this.loadSchemes({
      light: _designTokens.DesignTokens,
      dark: _designTokensDM.DesignTokensDM
    });
    _scheme.default.addChangeListener(() => {
      Object.assign(this, _scheme.default.getScheme());
    });
  }
  /**
   * Load custom set of colors
   * arguments:
   * colors - map of keys and colors values e.g {grey10: '#20303C', grey20: '#43515C'}
   */
  loadColors(colors) {
    Object.assign(this, colors);
  }
  /**
   * Load set of schemes for light/dark mode
   * arguments:
   * schemes - two sets of map of colors e.g {light: {screen: 'white'}, dark: {screen: 'black'}}
   */
  loadSchemes(schemes) {
    _scheme.default.loadSchemes(schemes);
    Object.assign(this, _scheme.default.getScheme());
  }

  /**
   * Get app's current color scheme
   */
  getScheme() {
    return _scheme.default.getSchemeType();
  }

  /**
   * Set color scheme for app
   * arguments:
   * scheme - color scheme e.g light/dark/default
   */
  setScheme(scheme) {
    _scheme.default.setScheme(scheme);
  }

  /**
   * Add alpha to hex or rgb color
   * arguments:
   * p1 - hex color / R part of RGB
   * p2 - opacity / G part of RGB
   * p3 - B part of RGB
   * p4 - opacity
   */

  rgba(p1, p2, p3, p4) {
    let hex;
    let opacity;
    let red;
    let green;
    let blue;

    // Handle design token PlatformColor object
    if (typeof p1 === 'object') {
      p1 = colorStringValue(p1);
    }
    if (arguments.length === 2 && typeof p1 === 'string') {
      hex = p1;
      opacity = p2;
      hex = validateHex(hex);
      red = parseInt(hex.substring(0, 2), 16);
      green = parseInt(hex.substring(2, 4), 16);
      blue = parseInt(hex.substring(4, 6), 16);
    } else if (arguments.length === 4 && typeof p1 === 'number') {
      red = validateRGB(p1);
      green = validateRGB(p2);
      blue = validateRGB(p3);
      opacity = p4;
    } else {
      console.error('Colors.rgba fail due to invalid arguments');
      return undefined;
    }
    return `rgba(${red}, ${green}, ${blue}, ${opacity})`;
  }
  getBackgroundKeysPattern() {
    return /^(bg-|background-)/;
  }
}
exports.Colors = Colors;
function colorStringValue(color) {
  return color?.toString();
}
function validateRGB(value) {
  if (isNaN(value) || value > 255 || value < 0) {
    throw new Error(`${value} is invalid rgb code, please use number between 0-255`);
  }
  return value;
}
function validateHex(value) {
  if (!/^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/.test(value)) {
    throw new Error(`${value} is invalid hex color`);
  }
  value = value.replace('#', '');
  if (value.length === 3) {
    value = threeDigitHexToSix(value);
  }
  return value;
}
function threeDigitHexToSix(value) {
  return value.replace(/./g, '$&$&');
}
const TypedColors = Colors;
const colorObject = new TypedColors();
var _default = exports.default = colorObject;
//# sourceMappingURL=colors.js.map