"use strict";

import { StyleSheet } from 'react-native';
import { Typography, Colors, BorderRadiuses, Spacings, ThemeManager } from "../style/index.js";
export const FLEX_KEY_PATTERN = /^flex(G|S)?(-\d*)?$/;
export const PADDING_KEY_PATTERN = new RegExp(`padding[LTRBHV]?-([0-9]*|${Spacings.getKeysPattern()})`);
export const MARGIN_KEY_PATTERN = new RegExp(`margin[LTRBHV]?-([0-9]*|${Spacings.getKeysPattern()})`);
export const ALIGNMENT_KEY_PATTERN = /(left|top|right|bottom|center|centerV|centerH|spread)/;
export const POSITION_KEY_PATTERN = /^abs([F|L|R|T|B|V|H])?$/;
const BACKGROUND_COLOR_KEYS_PATTERN = Colors.getBackgroundKeysPattern();
export const GAP_KEY_PATTERN = new RegExp(`gap-([0-9]*|${Spacings.getKeysPattern()})`);
const PADDING_VARIATIONS = {
  padding: 'padding',
  paddingL: 'paddingLeft',
  paddingT: 'paddingTop',
  paddingR: 'paddingRight',
  paddingB: 'paddingBottom',
  paddingH: 'paddingHorizontal',
  paddingV: 'paddingVertical'
};
const MARGIN_VARIATIONS = {
  margin: 'margin',
  marginL: 'marginLeft',
  marginT: 'marginTop',
  marginR: 'marginRight',
  marginB: 'marginBottom',
  marginH: 'marginHorizontal',
  marginV: 'marginVertical'
};
const STYLE_KEY_CONVERTERS = {
  flex: 'flex',
  flexG: 'flexGrow',
  flexS: 'flexShrink'
};
const POSITION_CONVERSIONS = {
  F: 'Fill',
  T: 'Top',
  B: 'Bottom',
  L: 'Left',
  R: 'Right',
  H: 'Horizontal',
  V: 'Vertical'
};

// TODO: migrate other modifiers to the same new structure as Margin modifier, using template literals

// TODO: This caused issue with with some typings that inherit this type
// export type MarginModifiers = Partial<{[key: `${MarginLiterals}-${number}`]: boolean}>;

export function extractAlignmentsValues(props) {
  const {
    row,
    center
  } = props;
  const alignments = row ? {
    flexDirection: 'row'
  } : {};
  const alignmentRules = row ? {
    justifyContent: ['left', 'right', 'centerH', 'spread'],
    alignItems: ['top', 'bottom', 'centerV']
  } : {
    justifyContent: ['top', 'bottom', 'centerV', 'spread'],
    alignItems: ['left', 'right', 'centerH']
  };
  for (const attr in alignmentRules) {
    const attribute = attr;
    const positions = alignmentRules[attribute];
    for (const position of positions) {
      if (props[position]) {
        if (position === 'left' || position === 'top') {
          alignments[attribute] = 'flex-start';
        } else if (position === 'right' || position === 'bottom') {
          alignments[attribute] = 'flex-end';
        } else if (position === 'centerH' || position === 'centerV') {
          alignments[attribute] = 'center';
        } else if (position === 'spread') {
          alignments[attribute] = 'space-between';
        }
      }
    }
  }
  if (center) {
    alignments.justifyContent = 'center';
    alignments.alignItems = 'center';
  }
  return alignments;
}
export function extractAccessibilityProps(props) {
  const accessibilityProps = {};
  for (let i = 0, len = props.length; i < len; i++) {
    const [propKey, propValue] = props[i];
    if (propValue === undefined) {
      continue;
    }
    if (/.*ccessib.*/.test(propKey)) {
      accessibilityProps[propKey] = propValue;
    }
  }
  return accessibilityProps;
}
export const ANIMATION_PROPS = ['animation', 'duration', 'delay', 'direction', 'easing', 'iterationCount', 'transition', 'onAnimationBegin', 'onAnimationEnd', 'useNativeDriver'];
export function extractAnimationProps(props) {
  const animationProps = {};
  ANIMATION_PROPS.forEach(prop => {
    if (props[prop]) {
      animationProps[prop] = props[prop];
    }
  });
  return animationProps;
}
export function extractModifierProps(props) {
  const patterns = [FLEX_KEY_PATTERN, PADDING_KEY_PATTERN, MARGIN_KEY_PATTERN, ALIGNMENT_KEY_PATTERN, GAP_KEY_PATTERN, Colors.getBackgroundKeysPattern()];
  const modifierProps = {};
  for (let i = 0, len = props.length; i < len; i++) {
    const [propKey, propValue] = props[i];
    if (propValue === undefined) {
      continue;
    }
    const isModifier = patterns.some(pattern => pattern.test(propKey));
    if (isModifier) {
      modifierProps[propKey] = propValue;
    }
  }
  return modifierProps;
}
export function extractComponentProps(component, props, ignoreProps = []) {
  const componentPropTypes = component.propTypes;
  const componentPropKeys = Object.keys(componentPropTypes);
  const componentProps = Object.keys(props).filter(key => componentPropKeys.includes(key) && !ignoreProps.includes(key)).reduce((acc, key) => {
    acc[key] = props[key];
    return acc;
  }, {});
  return componentProps;
}
export function getThemeProps(props, context, componentDisplayName = '') {
  const componentName = componentDisplayName;
  let themeProps;
  if (typeof ThemeManager.components[componentName] === 'function') {
    themeProps = ThemeManager.components[componentName](props, context);
  } else {
    themeProps = ThemeManager.components[componentName];
  }
  let forcedThemeProps;
  if (typeof ThemeManager.forcedThemeComponents[componentName] === 'function') {
    forcedThemeProps = ThemeManager.forcedThemeComponents[componentName](props, context);
  } else {
    forcedThemeProps = ThemeManager.forcedThemeComponents[componentName];
  }
  return {
    ...themeProps,
    ...props,
    ...forcedThemeProps
  };
}
export function generateModifiersStyle(options = {
  color: true,
  typography: true,
  backgroundColor: true,
  borderRadius: true,
  paddings: true,
  margins: true,
  alignments: true,
  flex: true,
  position: true,
  gap: false
}, props) {
  const boundProps = props;
  const style = {};
  for (const propKey in boundProps) {
    if (options.color) {
      const color = Colors[propKey];
      if (color) {
        style.color = color;
      }
    }
    if (options.typography) {
      if (Typography[propKey]) {
        style.typography = Object.assign({}, style.typography, Typography[propKey]);
      }
    }
    if (options.backgroundColor) {
      if (BACKGROUND_COLOR_KEYS_PATTERN.test(propKey)) {
        const key = propKey.replace(BACKGROUND_COLOR_KEYS_PATTERN, '');
        let color = Colors[key];
        if (color) {
          style.backgroundColor = color;
        }
      }
    }
    if (options.borderRadius) {
      if (BorderRadiuses.getKeysPattern().test(propKey)) {
        style.borderRadius = BorderRadiuses[propKey];
      }
    }
    if (options.paddings) {
      if (PADDING_KEY_PATTERN.test(propKey)) {
        const [paddingKey, paddingValue] = propKey.split('-');
        const paddingVariation = PADDING_VARIATIONS[paddingKey];
        if (!isNaN(Number(paddingValue))) {
          style.paddings = {
            ...style.paddings,
            [paddingVariation]: Number(paddingValue)
          };
        } else if (Spacings.getKeysPattern().test(paddingValue)) {
          style.paddings = {
            ...style.paddings,
            [paddingVariation]: Spacings[paddingValue]
          };
        }
      }
    }
    if (options.margins) {
      if (MARGIN_KEY_PATTERN.test(propKey)) {
        const [marginKey, marginValue] = propKey.split('-');
        const marginVariation = MARGIN_VARIATIONS[marginKey];
        if (!isNaN(Number(marginValue))) {
          style.margins = {
            ...style.margins,
            [marginVariation]: Number(marginValue)
          };
        } else if (Spacings.getKeysPattern().test(marginValue)) {
          style.margins = {
            ...style.margins,
            [marginVariation]: Spacings[marginValue]
          };
        }
      }
    }
    if (options.flex) {
      if (FLEX_KEY_PATTERN.test(propKey)) {
        const [flexKey, flexValue] = propKey.split('-');
        const convertedFlexKey = STYLE_KEY_CONVERTERS[flexKey];
        const flexValueAsNumber = isNaN(Number(flexValue)) ? 1 : Number(flexValue);
        style.flexStyle = {
          [convertedFlexKey]: flexValueAsNumber
        };
      }
    }
    if (options.position) {
      if (POSITION_KEY_PATTERN.test(propKey)) {
        const positionVariationKey = propKey.replace('abs', '');
        if (positionVariationKey) {
          const positionVariation = POSITION_CONVERSIONS[positionVariationKey];
          const styleKey = `absolute${positionVariation}`;
          style.positionStyle = Object.assign({}, style.positionStyle, styles[styleKey]);
        }
        style.positionStyle = Object.assign({}, style.positionStyle, styles.absolute);
      }
    }
    if (options.gap) {
      if (GAP_KEY_PATTERN.test(propKey)) {
        const [, gapValue] = propKey.split('-');
        const parsedNumber = Number(gapValue);
        if (!isNaN(parsedNumber)) {
          style.gap = parsedNumber;
        } else if (Spacings.getKeysPattern().test(gapValue)) {
          style.gap = Spacings[gapValue];
        }
      }
    }
  }
  if (options.alignments) {
    style.alignments = extractAlignmentsValues(boundProps);
  }
  return style;
}
const styles = StyleSheet.create({
  absolute: {
    position: 'absolute'
  },
  absoluteFill: StyleSheet.absoluteFillObject,
  absoluteTop: {
    position: 'absolute',
    top: 0
  },
  absoluteBottom: {
    position: 'absolute',
    bottom: 0
  },
  absoluteLeft: {
    position: 'absolute',
    left: 0
  },
  absoluteRight: {
    position: 'absolute',
    right: 0
  },
  absoluteVertical: {
    position: 'absolute',
    top: 0,
    bottom: 0
  },
  absoluteHorizontal: {
    position: 'absolute',
    left: 0,
    right: 0
  }
});
//# sourceMappingURL=modifiers.js.map