"use strict";

import { forwardRef, useMemo } from 'react';
import { I18nManager, Platform, Text as RNText, StyleSheet } from 'react-native';
import useThemeProps from "../../hooks/useThemeProps.js";
import useModifiers from "../../hooks/useModifiers.js";
import { Colors } from "../../style/index.js";
import { jsx as _jsx } from "react/jsx-runtime";
const modifiersOptions = {
  color: true,
  margins: true,
  typography: true,
  backgroundColor: true,
  flex: true
};
var writingDirectionTypes = /*#__PURE__*/function (writingDirectionTypes) {
  writingDirectionTypes["RTL"] = "rtl";
  writingDirectionTypes["LTR"] = "ltr";
  return writingDirectionTypes;
}(writingDirectionTypes || {});
/**
 * @description: A wrapper for Text component with extra functionality like modifiers support
 * @extends: Text
 * @extendsLink: https://reactnative.dev/docs/text
 * @modifiers: margins, color, typography
 * @example: https://github.com/wix/react-native-ui-lib/blob/master/demo/src/screens/componentScreens/TextScreen.tsx
 * @image: https://github.com/wix/react-native-ui-lib/blob/master/demo/showcase/Text/Modifiers.png?raw=true, https://github.com/wix/react-native-ui-lib/blob/master/demo/showcase/Text/Transformation.png?raw=true, https://github.com/wix/react-native-ui-lib/blob/master/demo/showcase/Text/Highlights.png?raw=true
 */
function Text(props, ref) {
  const themeProps = useThemeProps(props, 'Text');
  const {
    style,
    flex: propsFlex,
    children,
    as,
    color: propsColor,
    center,
    uppercase,
    lowerCase,
    underline,
    ...others
  } = themeProps;
  const {
    typography,
    color: modifierColor,
    backgroundColor,
    margins,
    flexStyle
  } = useModifiers(themeProps, modifiersOptions);
  const color = propsColor || modifierColor;
  const TextContainer = useMemo(() => {
    return as ?? RNText;
  }, [as]);
  const textStyle = [styles.container, typography, color && {
    color
  }, backgroundColor && {
    backgroundColor
  }, flexStyle, margins, center && styles.centered, uppercase && styles.uppercase, underline && styles.underline, style];
  return /*#__PURE__*/_jsx(TextContainer, {
    ...others,
    style: textStyle,
    ref: ref,
    children: children
  });
}
const styles = StyleSheet.create({
  container: {
    backgroundColor: 'transparent',
    color: Colors.$textDefault,
    ...(Platform.OS === 'ios' ? {
      writingDirection: I18nManager.isRTL ? writingDirectionTypes.RTL : writingDirectionTypes.LTR
    } : {
      textAlign: 'left'
    })
  },
  centered: {
    textAlign: 'center'
  },
  uppercase: {
    textTransform: 'uppercase'
  },
  underline: {
    textDecorationLine: 'underline'
  },
  highlight: {
    color: Colors.grey30
  },
  notHighlight: {
    color: undefined
  }
});
export default /*#__PURE__*/forwardRef(Text);
//# sourceMappingURL=text.js.map