"use strict";

import { Appearance, Platform, PlatformColor } from 'react-native';
import Config from "../commons/Config.js";
class Scheme {
  currentScheme = Config.appScheme;
  schemes = {
    light: {},
    dark: {}
  };
  changeListeners = [];
  constructor() {
    Appearance.addChangeListener(() => {
      if (this.currentScheme === 'default') {
        this.broadcastSchemeChange();
      }
    });
  }
  broadcastSchemeChange() {
    this.changeListeners.forEach(listener => listener(this.getSchemeType()));
  }

  /**
   * Get app's current color scheme
   */
  getSchemeType() {
    const scheme = this.currentScheme === 'default' ? Appearance.getColorScheme() : this.currentScheme;
    return scheme ?? 'light';
  }

  /**
   * Whether the app's scheme is 'dark', i.e. is on dark mode
   */
  isDarkMode() {
    return this.getSchemeType() === 'dark';
  }

  /**
   * Set color scheme for app
   * arguments:
   * scheme - color scheme e.g light/dark/default
   */
  setScheme(scheme) {
    const prevSchemeType = this.getSchemeType();
    if (!['light', 'dark', 'default'].includes(scheme)) {
      throw new Error(`${scheme} is invalid colorScheme, please use 'light' | 'dark' | 'default'`);
    }
    this.currentScheme = scheme;
    if (prevSchemeType !== this.getSchemeType()) {
      this.broadcastSchemeChange();
    }
  }

  /**
   * Load set of schemes for light/dark mode
   * arguments:
   * schemes - two sets of map of colors e.g {light: {screen: 'white'}, dark: {screen: 'black'}}
   */
  loadSchemes(schemes) {
    const lightSchemeKeys = Object.keys(schemes.light);
    const darkSchemeKeys = Object.keys(schemes.dark);
    const missingKeys = lightSchemeKeys.filter(key => !darkSchemeKeys.includes(key));
    if (missingKeys.length > 0) {
      throw new Error(`There is a mismatch in scheme keys: ${missingKeys.join(', ')}`);
    }
    const platformColorsSchemes = JSON.parse(JSON.stringify(schemes));
    for (const schemeKey in schemes) {
      const scheme = schemes[schemeKey];
      for (const colorKey in scheme) {
        Object.defineProperty(platformColorsSchemes[schemeKey], colorKey, {
          get: () => {
            let color = scheme[colorKey];
            if (Config.usePlatformColors) {
              if (Platform.OS === 'android') {
                // Remove the $ prefix cause it's not allowed in Android and add the @color prefix
                color = PlatformColor(`@color/${colorKey.replace(/^[$]/, '')}`);
              } else {
                color = PlatformColor(colorKey);
              }
              // Get the original hex string value by calling toString()
              color.toString = () => schemes[this.getSchemeType()][colorKey];
            }
            return color;
          }
        });
      }
    }
    this.schemes = platformColorsSchemes;
  }

  /**
   * Retrieve scheme by current scheme type
   */
  getScheme() {
    return this.schemes[this.getSchemeType()];
  }

  /**
   * Add a change scheme event listener
   */
  addChangeListener(listener) {
    this.changeListeners.push(listener);
  }

  /**
   * Remove a change scheme event listener
   * arguments:
   * listener - listener reference to remove
   */
  removeChangeListener(listener) {
    this.changeListeners = this.changeListeners.filter(changeListener => changeListener !== listener);
  }
}
export default new Scheme();
//# sourceMappingURL=scheme.js.map